@@ -60,7 +60,7 @@ static const QString s_string_entry6{};
 static const char s_utf8bit_entry[] = "Hello äöü";
 static const QString s_translated_string_entry1{QStringLiteral("bonjour")};
 static const QByteArray s_bytearray_entry{"\x00\xff\x7f\x3c abc\x00\x00", 10};
-static const char s_escapekey[] = " []\0017[]==]";
+static const char s_escapekey[] = " []\0017[]==\\]";
 static const char s_escape_entry[] = "[]\170[]]=3=]\\] ";
 static const double s_double_entry{123456.78912345};
 static const float s_float_entry{123.567f};
@@ -262,25 +262,6 @@ void KConfigTest::cleanupTestCase()
     }
 }
 
-static QList<QByteArray> readLinesFrom(const QString &path)
-{
-    QFile file(path);
-    const bool opened = file.open(QIODevice::ReadOnly | QIODevice::Text);
-    QList<QByteArray> lines;
-    if (!opened) {
-        qWarning() << "Failed to open" << path;
-        return lines;
-    }
-    QByteArray line;
-    do {
-        line = file.readLine();
-        if (!line.isEmpty()) {
-            lines.append(line);
-        }
-    } while (!line.isEmpty());
-    return lines;
-}
-
 static const QString s_defaultArg = s_test_subdir + QLatin1String("kconfigtest");
 static QList<QByteArray> readLines(const QString &fileName = s_defaultArg)
 {
@@ -516,7 +497,7 @@ void KConfigTest::testPath()
     qputenv("WITHSLASH", "/a/");
     {
         QFile file(m_testConfigDir + QLatin1String("/pathtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Test Group]\n"
             << "homePath=$HOME/foo\n"
@@ -577,7 +558,7 @@ void KConfigTest::testPathQtHome()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/pathtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Test Group]\n"
             << "dataDir[$e]=$QT_DATA_HOME/kconfigtest\n"
@@ -1017,7 +998,7 @@ void KConfigTest::testMerge()
     {
         // simulate writing by another process
         QFile file(m_testConfigDir + QLatin1String("/mergetest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Merged Group]\n"
             << "entry1=Testing\n"
@@ -1043,7 +1024,7 @@ void KConfigTest::testMerge()
               << "entry[es]=Spanish\n"
               << "entry[fr]=French\n";
         QFile file(m_testConfigDir + QLatin1String("/mergetest"));
-        file.open(QIODevice::ReadOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::ReadOnly | QIODevice::Text));
         for (const QByteArray &line : std::as_const(lines)) {
             QCOMPARE(line, file.readLine());
         }
@@ -1054,7 +1035,7 @@ void KConfigTest::testImmutable()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/immutabletest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[$i]\n"
             << "entry1=Testing\n"
@@ -1078,7 +1059,7 @@ void KConfigTest::testOptionOrder()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/doubleattrtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[group3]\n"
             << "entry2=unlocalized\n"
@@ -1104,7 +1085,7 @@ void KConfigTest::testOptionOrder()
               << "entry2[de_DE][$i]=t2\n";
 
         QFile file(m_testConfigDir + QLatin1String("/doubleattrtest"));
-        file.open(QIODevice::ReadOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::ReadOnly | QIODevice::Text));
         for (const QByteArray &line : std::as_const(lines)) {
             QCOMPARE(line, file.readLine());
         }
@@ -1454,7 +1435,9 @@ void KConfigTest::testDirtyOnEqualOverdo()
 void KConfigTest::testCreateDir()
 {
     // Test auto-creating the parent directory when needed (KConfigIniBackend::createEnclosing)
-    const QString kdehome = QDir::home().canonicalPath() + QLatin1String("/.kde-unit-test");
+    QTemporaryDir tmpDir(QDir::tempPath() + QStringLiteral("/fakehome_XXXXXX"));
+    QVERIFY(tmpDir.isValid());
+    const QString kdehome = tmpDir.path() + QLatin1String("/.kde-unit-test");
     const QString subdir = kdehome + QLatin1String("/newsubdir");
     const QString file = subdir + QLatin1String("/foo.desktop");
     QFile::remove(file);
@@ -1548,7 +1531,9 @@ void KConfigTest::testLocaleConfig()
 void KConfigTest::testDeleteWhenLocalized()
 {
     // Initialize the testdata
-    const QString subdir = QDir::home().canonicalPath() + QLatin1String("/.kde-unit-test/");
+    QTemporaryDir tmpDir(QDir::tempPath() + QStringLiteral("/fakehome_XXXXXX"));
+    QVERIFY(tmpDir.isValid());
+    const QString subdir = tmpDir.path() + QLatin1String("/.kde-unit-test/");
     QDir().mkpath(subdir);
     const QString file = subdir + QLatin1String("/localized_delete.test");
     QFile::remove(file);
@@ -1916,7 +1901,89 @@ void KConfigTest::testMoveValuesTo()
 
     targetGroup.sync();
     QFile targetReadFile(targetFile.fileName());
-    targetReadFile.open(QFile::ReadOnly);
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
+    QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
+}
+
+void KConfigTest::testMoveValuesToWithNestedGroup()
+{
+    QTemporaryFile file;
+    QVERIFY(file.open());
+    // Prepare kdeglobals
+    {
+        KConfig glob(QStringLiteral("kdeglobals"));
+        KConfigGroup general(&glob, QStringLiteral("TestGroup"));
+        general.writeEntry("GlobalKey", "PlsDeleteMe");
+        QVERIFY(glob.sync());
+    }
+
+    KConfigGroup grp = KSharedConfig::openConfig(file.fileName())->group(QStringLiteral("TestGroup")).group(QStringLiteral("Nested"));
+
+    grp.writeEntry("test1", "first_value");
+    grp.writeEntry("test_empty", "");
+    grp.writeEntry("other", "other_value");
+    grp.writePathEntry("my_path", QStringLiteral("~/somepath"));
+    // because this key is from the global file it should be explicitly deleted
+    grp.deleteEntry("GlobalKey");
+
+    QTemporaryFile targetFile;
+    QVERIFY(targetFile.open());
+    targetFile.close();
+    KConfigGroup targetGroup =
+        KSharedConfig::openConfig(targetFile.fileName(), KConfig::SimpleConfig)->group(QStringLiteral("MoveToGroup")).group(QStringLiteral("AnotherNested"));
+
+    grp.moveValuesTo({"test1", "test_empty", "does_not_exist", "my_path", "GlobalKey"}, targetGroup);
+    QVERIFY(grp.config()->isDirty());
+    QVERIFY(targetGroup.config()->isDirty());
+
+    QCOMPARE(grp.keyList(), QStringList{QStringLiteral("other")});
+    QStringList expectedKeyList{QStringLiteral("my_path"), QStringLiteral("test1"), QStringLiteral("test_empty")};
+
+    QCOMPARE(targetGroup.keyList(), expectedKeyList);
+    QCOMPARE(targetGroup.readEntry("test1"), QStringLiteral("first_value"));
+
+    targetGroup.sync();
+    QFile targetReadFile(targetFile.fileName());
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
+    QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
+}
+
+void KConfigTest::testMoveAllValuesTo()
+{
+    QTemporaryFile file;
+    QVERIFY(file.open());
+    // Prepare kdeglobals
+    {
+        KConfig glob(QStringLiteral("kdeglobals"));
+        KConfigGroup general(&glob, QStringLiteral("TestGroup"));
+        general.writeEntry("GlobalKey", "PlsDeleteMe");
+        QVERIFY(glob.sync());
+    }
+
+    KConfigGroup grp = KSharedConfig::openConfig(file.fileName())->group(QStringLiteral("TestGroup"));
+
+    grp.writeEntry("test1", "first_value");
+    grp.writeEntry("test_empty", "");
+    grp.writePathEntry("my_path", QStringLiteral("~/somepath"));
+    // because this key is from the global file it should be explicitly deleted
+    grp.deleteEntry("GlobalKey");
+
+    QTemporaryFile targetFile;
+    QVERIFY(targetFile.open());
+    targetFile.close();
+    KConfigGroup targetGroup = KSharedConfig::openConfig(targetFile.fileName(), KConfig::SimpleConfig)->group(QStringLiteral("MoveToGroup"));
+
+    grp.moveValuesTo(targetGroup);
+    QVERIFY(grp.config()->isDirty());
+    QVERIFY(targetGroup.config()->isDirty());
+
+    QStringList expectedKeyList{QStringLiteral("my_path"), QStringLiteral("test1"), QStringLiteral("test_empty")};
+    QCOMPARE(targetGroup.keyList(), expectedKeyList);
+    QCOMPARE(targetGroup.readEntry("test1"), QStringLiteral("first_value"));
+
+    targetGroup.sync();
+    QFile targetReadFile(targetFile.fileName());
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
     QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
 }
 
